@@ -1,21 +1,24 @@
 #![allow(clippy::manual_map)]
+use std::fmt::Write;
+
+use log::{info, warn};
+use once_cell::sync::Lazy;
+use regex::{Captures, Regex};
+
 use super::{gi_docgen, LocationInObject};
 use crate::{
     analysis::functions::Info,
     library::{FunctionKind, TypeId},
     nameutil, Env,
 };
-use log::{info, warn};
-use once_cell::sync::Lazy;
-use regex::{Captures, Regex};
-use std::fmt::Write;
 
 const LANGUAGE_SEP_BEGIN: &str = "<!-- language=\"";
 const LANGUAGE_SEP_END: &str = "\" -->";
 const LANGUAGE_BLOCK_BEGIN: &str = "|[";
 const LANGUAGE_BLOCK_END: &str = "\n]|";
 
-// A list of function names that are ignored when warning about a "not found function"
+// A list of function names that are ignored when warning about a "not found
+// function"
 const IGNORE_C_WARNING_FUNCS: [&str; 6] = [
     "g_object_unref",
     "g_object_ref",
@@ -73,9 +76,9 @@ fn get_language<'a>(entry: &'a str, out: &mut String) -> &'a str {
     if let (_, Some(after)) = try_split(entry, LANGUAGE_SEP_BEGIN) {
         if let (before, Some(after)) = try_split(after, LANGUAGE_SEP_END) {
             if !["text", "rust"].contains(&before) {
-                write!(out, "\n\n**⚠️ The following code is in {} ⚠️**", before).unwrap();
+                write!(out, "\n\n**⚠️ The following code is in {before} ⚠️**").unwrap();
             }
-            write!(out, "\n\n```{}", before).unwrap();
+            write!(out, "\n\n```{before}").unwrap();
             return after;
         }
     }
@@ -111,9 +114,9 @@ fn format(
                 if !["text", "rust", "xml", "css", "json", "html"].contains(&lang)
                     && after.lines().count() > 1
                 {
-                    write!(ret, "**⚠️ The following code is in {0} ⚠️**\n\n", lang).unwrap();
+                    write!(ret, "**⚠️ The following code is in {lang} ⚠️**\n\n").unwrap();
                 }
-                writeln!(ret, "```{}", lang).unwrap();
+                writeln!(ret, "```{lang}").unwrap();
 
                 if let (before, Some(after)) = try_split(after, "```") {
                     ret.push_str(before);
@@ -145,7 +148,7 @@ fn replace_symbols(
             symbol_name => match &caps[1] {
                 // Opt-in only for the %SYMBOLS, @/# causes breakages
                 "%" => find_constant_or_variant_wrapper(symbol_name, env, in_type),
-                s => panic!("Unknown symbol prefix `{}`", s),
+                s => panic!("Unknown symbol prefix `{s}`"),
             },
         });
         let out = GDK_GTK.replace_all(&out, |caps: &Captures<'_>| {
@@ -164,8 +167,9 @@ static GI_DOCGEN_SYMBOL: Lazy<Regex> =
 static FUNCTION: Lazy<Regex> =
     Lazy::new(|| Regex::new(r"([@#%])?(\w+\b[:.]+)?(\b[a-z0-9_]+)\(\)").unwrap());
 // **note**
-// The optional . at the end is to make the regex more relaxed for some weird broken cases on gtk3's docs
-// it doesn't hurt other docs so please don't drop it
+// The optional . at the end is to make the regex more relaxed for some weird
+// broken cases on gtk3's docs it doesn't hurt other docs so please don't drop
+// it
 static GDK_GTK: Lazy<Regex> = Lazy::new(|| {
     Regex::new(r"`([^\(:])?((G[dts]k|Pango|cairo_|graphene_|Adw|Hdy|GtkSource)\w+\b)(\.)?`")
         .unwrap()
@@ -199,7 +203,7 @@ fn replace_c_types(
                     let method_name = member_path.trim_start_matches('.');
                     find_member(symbol_name, method_name, env, in_type).unwrap_or_else(|| {
                         info!("`#{}` not found as method", symbol_name);
-                        format!("`{}{}`", symbol_name, member_path)
+                        format!("`{symbol_name}{member_path}`")
                     })
                 } else if let Some(type_) = find_type(symbol_name, env) {
                     type_
@@ -213,7 +217,7 @@ fn replace_c_types(
                     constant_or_variant
                 } else {
                     info!("Type `#{}` not found", symbol_name);
-                    format!("`{}`", symbol_name)
+                    format!("`{symbol_name}`")
                 }
             }
             "@" => {
@@ -235,10 +239,10 @@ fn replace_c_types(
                     function
                 } else {
                     // `@` is often used to refer to fields and function parameters.
-                    format!("`{}`", symbol_name)
+                    format!("`{symbol_name}`")
                 }
             }
-            s => panic!("Unknown symbol prefix `{}`", s),
+            s => panic!("Unknown symbol prefix `{s}`"),
         },
     });
     let out = GDK_GTK.replace_all(&out, |caps: &Captures<'_>| {
@@ -257,7 +261,7 @@ fn find_constant_or_variant_wrapper(
 ) -> String {
     find_constant_or_variant(symbol_name, env, in_type).unwrap_or_else(|| {
         info!("Constant or variant `%{}` not found", symbol_name);
-        format!("`{}`", symbol_name)
+        format!("`{symbol_name}`")
     })
 }
 
@@ -384,7 +388,6 @@ fn find_method_or_function_by_ctype(
     in_type: Option<(&TypeId, Option<LocationInObject>)>,
 ) -> Option<String> {
     find_method_or_function(
-        name,
         env,
         in_type,
         |f| f.glib_name == name,
@@ -393,17 +396,21 @@ fn find_method_or_function_by_ctype(
         |r| c_type.map_or(true, |t| r.type_(&env.library).c_type == t),
         |r| c_type.map_or(true, |t| r.type_(&env.library).c_type == t),
         c_type.map_or(false, |t| t.ends_with("Class")),
+        false,
     )
 }
 
-/// Find a function in all the possible items, if not found return the original name surrounded with backticks.
-/// A function can either be a struct/interface/record method, a global function or maybe a virtual function
+/// Find a function in all the possible items, if not found return the original
+/// name surrounded with backticks. A function can either be a
+/// struct/interface/record method, a global function or maybe a virtual
+/// function
 ///
 /// This function is generic so it can be de-duplicated between a
-/// - [`find_method_or_function_by_ctype()`] where the object/records are looked by their C name
-/// - [`gi_docgen::find_method_or_function_by_name()`] where the object/records are looked by their name
+/// - [`find_method_or_function_by_ctype()`] where the object/records are looked
+///   by their C name
+/// - [`gi_docgen::find_method_or_function_by_name()`] where the object/records
+///   are looked by their name
 pub(crate) fn find_method_or_function(
-    name: &str,
     env: &Env,
     in_type: Option<(&TypeId, Option<LocationInObject>)>,
     search_fn: impl Fn(&crate::analysis::functions::Info) -> bool + Copy,
@@ -412,14 +419,40 @@ pub(crate) fn find_method_or_function(
     search_enum: impl Fn(&crate::analysis::enums::Info) -> bool + Copy,
     search_flag: impl Fn(&crate::analysis::flags::Info) -> bool + Copy,
     is_class_method: bool,
+    is_virtual_method: bool,
 ) -> Option<String> {
-    if is_class_method {
-        info!("Class methods are not supported yet `{}`", name);
-        return None;
-    }
-
+    if is_virtual_method {
+        if let Some((obj_info, fn_info)) = env
+            .analysis
+            .find_object_by_virtual_method(env, search_obj, search_fn)
+        {
+            Some(gen_object_fn_doc_link(
+                obj_info,
+                fn_info,
+                env,
+                in_type,
+                &obj_info.name,
+            ))
+        } else {
+            None
+        }
+    } else if is_class_method {
+        if let Some((record_info, fn_info)) =
+            env.analysis
+                .find_record_by_function(env, search_record, search_fn)
+        {
+            let object = env.config.objects.get(&record_info.full_name);
+            let visible_parent = object
+                .and_then(|o| o.trait_name.clone())
+                .unwrap_or_else(|| format!("{}Ext", record_info.name));
+            let parent = format!("subclass::prelude::{}", visible_parent);
+            let is_self = in_type == Some((&record_info.type_id, None));
+            Some(fn_info.doc_link(Some(&parent), Some(&visible_parent), is_self))
+        } else {
+            None
+        }
     // if we can find the function in an object
-    if let Some((obj_info, fn_info)) = env
+    } else if let Some((obj_info, fn_info)) = env
         .analysis
         .find_object_by_function(env, search_obj, search_fn)
     {
@@ -493,7 +526,14 @@ pub(crate) fn gen_object_fn_doc_link(
     let sym = symbols.by_tid(obj_info.type_id).unwrap();
     let is_self = in_type == Some((&obj_info.type_id, Some(obj_info.function_location(fn_info))));
 
-    if fn_info.kind == FunctionKind::Method {
+    if fn_info.kind == FunctionKind::VirtualMethod || fn_info.kind == FunctionKind::ClassMethod {
+        let (type_name, visible_type_name) = obj_info.generate_doc_link_info(fn_info);
+        fn_info.doc_link(
+            Some(&sym.full_rust_name().replace(visible_name, &type_name)),
+            Some(&visible_type_name),
+            false,
+        )
+    } else if fn_info.kind == FunctionKind::Method {
         let (type_name, visible_type_name) = obj_info.generate_doc_link_info(fn_info);
 
         fn_info.doc_link(
@@ -518,9 +558,9 @@ pub(crate) fn gen_member_doc_link(
     let is_self = in_type == Some((&type_id, None));
 
     if is_self {
-        format!("[`{m}`][Self::{m}]", m = member_name)
+        format!("[`{member_name}`][Self::{member_name}]")
     } else {
-        format!("[`{s}::{m}`][crate::{s}::{m}]", s = sym, m = member_name)
+        format!("[`{sym}::{member_name}`][crate::{sym}::{member_name}]")
     }
 }
 
@@ -530,23 +570,23 @@ pub(crate) fn gen_const_doc_link(const_info: &crate::analysis::constants::Info)
 }
 
 pub(crate) fn gen_signal_doc_link(symbol: &str, signal: &str) -> String {
-    format!("`signal::{}::{}`", symbol, signal)
+    format!("[`{signal}`][struct@crate::{symbol}#{signal}]")
 }
 
 pub(crate) fn gen_property_doc_link(symbol: &str, property: &str) -> String {
-    format!("`property::{}::{}`", symbol, property)
+    format!("[`{property}`][struct@crate::{symbol}#{property}]")
 }
 
 pub(crate) fn gen_vfunc_doc_link(symbol: &str, vfunc: &str) -> String {
-    format!("`vfunc::{}::{}`", symbol, vfunc)
+    format!("`vfunc::{symbol}::{vfunc}`")
 }
 
 pub(crate) fn gen_callback_doc_link(callback: &str) -> String {
-    format!("`callback::{}", callback)
+    format!("`callback::{callback}")
 }
 
 pub(crate) fn gen_alias_doc_link(alias: &str) -> String {
-    format!("`alias::{}`", alias)
+    format!("`alias::{alias}`")
 }
 
 pub(crate) fn gen_symbol_doc_link(type_id: TypeId, env: &Env) -> String {
