@@ -1,8 +1,10 @@
+use std::{collections::HashMap, fs::File, io::prelude::*};
+
+use log::info;
+use toml::{self, value::Table, Value};
+
 use super::collect_versions;
 use crate::{config::Config, env::Env, file_saver::save_to_file, nameutil, version::Version};
-use log::info;
-use std::{collections::HashMap, fs::File, io::prelude::*};
-use toml::{self, value::Table, Value};
 
 pub fn generate(env: &Env) -> String {
     info!("Generating sys Cargo.toml for {}", env.config.library_name);
@@ -99,7 +101,7 @@ fn fill_in(root: &mut Table, env: &Env) {
     {
         let package = upsert_table(root, "package");
         set_string(package, "build", "build.rs");
-        //set_string(package, "version", "0.2.0");
+        // set_string(package, "version", "0.2.0");
     }
 
     {
@@ -139,38 +141,38 @@ fn fill_in(root: &mut Table, env: &Env) {
         let meta = upsert_table(meta, "system-deps");
 
         let ns = env.namespaces.main();
-        let lib_name = ns.package_name.as_ref().unwrap();
+        if let Some(lib_name) = ns.package_names.first() {
+            let meta = upsert_table(meta, nameutil::lib_name_to_toml(lib_name));
+            // Allow both the name and version of a system dep to be overridden by hand
+            meta.entry("name")
+                .or_insert_with(|| Value::String(lib_name.clone()));
+            meta.entry("version")
+                .or_insert_with(|| Value::String(env.config.min_cfg_version.to_string()));
 
-        let meta = upsert_table(meta, nameutil::lib_name_to_toml(lib_name));
-        // Allow both the name and version of a system dep to be overridden by hand
-        meta.entry("name")
-            .or_insert_with(|| Value::String(lib_name.clone()));
-        meta.entry("version")
-            .or_insert_with(|| Value::String(env.config.min_cfg_version.to_string()));
+            // Old version API
+            unset(meta, "feature-versions");
 
-        // Old version API
-        unset(meta, "feature-versions");
-
-        collect_versions(env)
-            .iter()
-            .filter(|(&v, _)| v > env.config.min_cfg_version)
-            .for_each(|(v, lib_version)| {
-                let version_section = upsert_table(meta, &v.to_feature());
-                // Allow system-deps version for this feature level to be overridden by hand
-                version_section
-                    .entry("version")
-                    .or_insert_with(|| Value::String(lib_version.to_string()));
-            });
+            collect_versions(env)
+                .iter()
+                .filter(|(&v, _)| v > env.config.min_cfg_version)
+                .for_each(|(v, lib_version)| {
+                    let version_section = upsert_table(meta, &v.to_feature());
+                    // Allow system-deps version for this feature level to be overridden by hand
+                    version_section
+                        .entry("version")
+                        .or_insert_with(|| Value::String(lib_version.to_string()));
+                });
+        }
     }
 
     {
-        // Small trick to prevent having double quotes around it since toml doesn't like having '.'
+        // Small trick to prevent having double quotes around it since toml doesn't like
+        // having '.'
         let docs_rs_metadata = upsert_table(root, "package");
         let docs_rs_metadata = upsert_table(docs_rs_metadata, "metadata");
         let docs_rs_metadata = upsert_table(docs_rs_metadata, "docs");
         let docs_rs_metadata = upsert_table(docs_rs_metadata, "rs");
-        let mut docs_rs_features = env.config.docs_rs_features.clone();
-        docs_rs_features.push("dox".to_owned());
+        let docs_rs_features = env.config.docs_rs_features.clone();
         docs_rs_metadata.insert(
             "features".to_string(),
             Value::Array(
@@ -181,15 +183,6 @@ fn fill_in(root: &mut Table, env: &Env) {
             ),
         );
     }
-
-    {
-        let features = upsert_table(root, "features");
-        let mut dox_features = Vec::new();
-        for ext_lib in &env.config.external_libraries {
-            dox_features.push(Value::String(format!("{}/dox", ext_lib.lib_name)));
-        }
-        features.insert("dox".to_string(), Value::Array(dox_features));
-    }
 }
 
 fn get_feature_dependencies(
@@ -211,7 +204,7 @@ fn get_feature_dependencies(
 fn get_crate_name(config: &Config, root: &Table) -> String {
     if let Some(Value::Table(lib)) = root.get("lib") {
         if let Some(Value::String(lib_name)) = lib.get("name") {
-            //Converting don't needed as library target names cannot contain hyphens
+            // Converting don't needed as library target names cannot contain hyphens
             return lib_name.clone();
         }
     }
