@@ -1,3 +1,13 @@
+use std::{
+    collections::HashMap,
+    fs,
+    ops::Index,
+    path::{Component, Path, PathBuf},
+    str::FromStr,
+};
+
+use log::warn;
+
 use super::{
     external_libraries::{read_external_libraries, ExternalLibrary},
     gobjects, WorkMode,
@@ -11,14 +21,6 @@ use crate::{
     nameutil::set_crate_name_overrides,
     version::Version,
 };
-use log::warn;
-use std::{
-    collections::HashMap,
-    fs,
-    ops::Index,
-    path::{Component, Path, PathBuf},
-    str::FromStr,
-};
 
 /// Performs canonicalization by removing `foo/../` and `./` components
 /// from `path`, without hitting the file system. It does not turn relative
@@ -118,6 +120,10 @@ pub struct Config {
     pub extra_versions: Vec<Version>,
     pub lib_version_overrides: HashMap<Version, Version>,
     pub feature_dependencies: HashMap<Version, Vec<String>>,
+    /// An url that will be inserted into the docs as link that links
+    /// to another doc source, for example when builds on docs.rs
+    /// are limited due to license issues.
+    pub external_docs_url: Option<String>,
 }
 
 impl Config {
@@ -132,7 +138,7 @@ impl Config {
         make_backup: bool,
         show_statistics: bool,
         disable_format: bool,
-    ) -> Result<Config, String>
+    ) -> Result<Self, String>
     where
         S: Into<Option<&'a str>>,
         W: Into<Option<WorkMode>>,
@@ -232,7 +238,7 @@ impl Config {
         };
         if work_mode == WorkMode::Normal && auto_path.exists() {
             std::fs::remove_dir_all(&auto_path)
-                .map_err(|e| format!("remove_dir_all failed: {:?}", e))?;
+                .map_err(|e| format!("remove_dir_all failed: {e:?}"))?;
         }
 
         let doc_target_path: PathBuf = match doc_target_path.into() {
@@ -344,8 +350,9 @@ impl Config {
         let extra_versions = read_extra_versions(&toml)?;
         let lib_version_overrides = read_lib_version_overrides(&toml)?;
         let feature_dependencies = read_feature_dependencies(&toml)?;
+        let external_docs_url = read_external_docs_url(&toml)?;
 
-        Ok(Config {
+        Ok(Self {
             work_mode,
             girs_dirs,
             girs_version,
@@ -372,6 +379,7 @@ impl Config {
             extra_versions,
             lib_version_overrides,
             feature_dependencies,
+            external_docs_url,
         })
     }
 
@@ -425,7 +433,10 @@ fn read_toml<P: AsRef<Path>>(filename: P) -> Result<toml::Value, String> {
     let input = fs::read(&filename)
         .map_err(|e| format!("Failed to read file \"{:?}\": {}", filename.as_ref(), e))?;
 
-    toml::from_slice(&input).map_err(|e| {
+    let input = String::from_utf8(input)
+        .map_err(|e| format!("File is not valid UTF-8 \"{:?}\": {}", filename.as_ref(), e))?;
+
+    toml::from_str(&input).map_err(|e| {
         format!(
             "Invalid toml format in \"{}\": {}",
             filename.as_ref().display(),
@@ -527,6 +538,17 @@ fn read_feature_dependencies(toml: &toml::Value) -> Result<HashMap<Version, Vec<
     Ok(map)
 }
 
+fn read_external_docs_url(toml: &toml::Value) -> Result<Option<String>, String> {
+    Ok(
+        if let Some(value) = toml.lookup("options.external_docs_url") {
+            let value = value.as_result_str("options.external_docs_url")?;
+            Some(value.to_string())
+        } else {
+            None
+        },
+    )
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
