@@ -1,13 +1,15 @@
+use std::{
+    path::{Path, PathBuf},
+    str::FromStr,
+};
+
+use log::{trace, warn};
+
 use crate::{
     library::*,
     version::Version,
     xmlparser::{Element, XmlParser},
 };
-use log::{trace, warn};
-use std::{
-    path::{Path, PathBuf},
-    str::FromStr,
-};
 
 const EMPTY_CTYPE: &str = "/*EMPTY*/";
 
@@ -43,14 +45,14 @@ impl Library {
         parser: &mut XmlParser<'_>,
         libs: &mut Vec<String>,
     ) -> Result<(), String> {
-        let mut package = None;
+        let mut packages = Vec::new();
         let mut includes = Vec::new();
         parser.elements(|parser, elem| match elem.name() {
             "include" => {
                 match (elem.attr("name"), elem.attr("version")) {
                     (Some(name), Some(ver)) => {
                         if self.find_namespace(name).is_none() {
-                            let lib = format!("{}-{}", name, ver);
+                            let lib = format!("{name}-{ver}");
                             if libs.iter().any(|x| *x == lib) {
                                 return Err(format!(
                                     "`{}` includes itself (full path:`{}`)!",
@@ -69,16 +71,16 @@ impl Library {
                 Ok(())
             }
             "package" => {
-                // Take the first package element and ignore any other ones.
-                if package.is_none() {
-                    let name = elem.attr_required("name")?;
-                    package = Some(name.to_owned());
-                }
+                let name = elem.attr_required("name")?;
+                packages.push(name.to_owned());
                 Ok(())
             }
-            "namespace" => {
-                self.read_namespace(parser, elem, package.take(), std::mem::take(&mut includes))
-            }
+            "namespace" => self.read_namespace(
+                parser,
+                elem,
+                std::mem::take(&mut packages),
+                std::mem::take(&mut includes),
+            ),
             "attribute" => parser.ignore_element(),
             _ => Err(parser.unexpected_element(elem)),
         })?;
@@ -89,7 +91,7 @@ impl Library {
         &mut self,
         parser: &mut XmlParser<'_>,
         elem: &Element,
-        package: Option<String>,
+        packages: Vec<String>,
         c_includes: Vec<String>,
     ) -> Result<(), String> {
         let ns_name = elem.attr_required("name")?;
@@ -97,10 +99,19 @@ impl Library {
 
         {
             let ns = self.namespace_mut(ns_id);
-            ns.package_name = package;
+            ns.package_names = packages;
             ns.c_includes = c_includes;
             if let Some(s) = elem.attr("shared-library") {
-                ns.shared_library = s.split(',').map(String::from).collect();
+                ns.shared_library = s
+                    .split(',')
+                    .filter_map(|x| {
+                        if !x.is_empty() {
+                            Some(String::from(x))
+                        } else {
+                            None
+                        }
+                    })
+                    .collect();
             }
             if let Some(s) = elem.attr("identifier-prefixes") {
                 ns.identifier_prefixes = s.split(',').map(String::from).collect();
@@ -129,7 +140,7 @@ impl Library {
                 "function" => self.read_global_function(parser, ns_id, elem),
                 "constant" => self.read_constant(parser, ns_id, elem),
                 "alias" => self.read_alias(parser, ns_id, elem),
-                "function-macro" | "docsection" => parser.ignore_element(),
+                "boxed" | "function-macro" | "docsection" => parser.ignore_element(),
                 _ => {
                     warn!("<{} name={:?}>", elem.name(), elem.attr("name"));
                     parser.ignore_element()
@@ -169,6 +180,7 @@ impl Library {
         let mut properties = Vec::new();
         let mut impls = Vec::new();
         let mut fields = Vec::new();
+        let mut vfns = Vec::new();
         let mut doc = None;
         let mut doc_deprecated = None;
         let mut union_count = 1;
@@ -191,7 +203,9 @@ impl Library {
             "field" => self.read_field(parser, ns_id, elem).map(|f| {
                 fields.push(f);
             }),
-            "virtual-method" => parser.ignore_element(),
+            "virtual-method" => self
+                .read_virtual_method(parser, ns_id, elem)
+                .map(|v| vfns.push(v)),
             "doc" => parser.text().map(|t| doc = Some(t)),
             "doc-deprecated" => parser.text().map(|t| doc_deprecated = Some(t)),
             "source-position" => parser.ignore_element(),
@@ -201,12 +215,12 @@ impl Library {
                     let field_name = if let Some(field_name) = elem.attr("name") {
                         field_name.into()
                     } else {
-                        format!("u{}", union_count)
+                        format!("u{union_count}")
                     };
 
                     u = Union {
-                        name: format!("{}_{}", class_name, field_name),
-                        c_type: Some(format!("{}_{}", c_type, field_name)),
+                        name: format!("{class_name}_{field_name}"),
+                        c_type: Some(format!("{c_type}_{field_name}")),
                         ..u
                     };
 
@@ -237,6 +251,7 @@ impl Library {
             glib_get_type: get_type.into(),
             fields,
             functions: fns,
+            virtual_methods: vfns,
             signals,
             properties,
             parent,
@@ -283,6 +298,8 @@ impl Library {
             parser.ignore_element()?;
             return Ok(None);
         }
+        let is_class_record = record_name.ends_with("Class");
+
         let c_type = elem.attr_required("type")?;
         let symbol_prefix = elem.attr("symbol-prefix").map(ToOwned::to_owned);
         let get_type = elem.attr("get-type").map(ToOwned::to_owned);
@@ -307,19 +324,19 @@ impl Library {
                     let field_name = if let Some(field_name) = elem.attr("name") {
                         field_name.into()
                     } else {
-                        format!("u{}", union_count)
+                        format!("u{union_count}")
                     };
 
                     u = Union {
                         name: format!(
                             "{}{}_{}",
-                            parent_name_prefix.map_or_else(String::new, |s| { format!("{}_", s) }),
+                            parent_name_prefix.map_or_else(String::new, |s| { format!("{s}_") }),
                             record_name,
                             field_name
                         ),
                         c_type: Some(format!(
                             "{}{}_{}",
-                            parent_ctype_prefix.map_or_else(String::new, |s| { format!("{}_", s) }),
+                            parent_ctype_prefix.map_or_else(String::new, |s| { format!("{s}_") }),
                             c_type,
                             field_name
                         )),
@@ -373,9 +390,18 @@ impl Library {
             name: record_name.into(),
             c_type: c_type.into(),
             glib_get_type: get_type,
+            functions: if is_class_record && gtype_struct_for.is_some() {
+                fns.into_iter()
+                    .map(|mut f| {
+                        f.kind = FunctionKind::ClassMethod;
+                        f
+                    })
+                    .collect::<Vec<_>>()
+            } else {
+                fns
+            },
             gtype_struct_for: gtype_struct_for.map(|s| s.into()),
             fields,
-            functions: fns,
             version,
             deprecated_version,
             doc,
@@ -452,19 +478,19 @@ impl Library {
                 let field_name = if let Some(field_name) = elem.attr("name") {
                     field_name.into()
                 } else {
-                    format!("s{}", struct_count)
+                    format!("s{struct_count}")
                 };
 
                 r = Record {
                     name: format!(
                         "{}{}_{}",
-                        parent_name_prefix.map_or_else(String::new, |s| { format!("{}_", s) }),
+                        parent_name_prefix.map_or_else(String::new, |s| { format!("{s}_") }),
                         union_name,
                         field_name
                     ),
                     c_type: format!(
                         "{}{}_{}",
-                        parent_ctype_prefix.map_or_else(String::new, |s| { format!("{}_", s) }),
+                        parent_ctype_prefix.map_or_else(String::new, |s| { format!("{s}_") }),
                         c_type,
                         field_name
                     ),
@@ -502,6 +528,77 @@ impl Library {
         })
     }
 
+    fn read_virtual_method(
+        &mut self,
+        parser: &mut XmlParser,
+        ns_id: u16,
+        elem: &Element,
+    ) -> Result<Function, String> {
+        let method_name = elem.attr_required("name")?;
+        let version = self.read_version(parser, ns_id, elem)?;
+        let deprecated_version = self.read_deprecated_version(parser, ns_id, elem)?;
+        let c_identifier = elem.attr("identifier").or_else(|| elem.attr("name"));
+        let mut params = Vec::new();
+        let mut ret = None;
+        let mut doc = None;
+        let mut doc_deprecated = None;
+
+        parser.elements(|parser, elem| match elem.name() {
+            "parameters" => self
+                .read_parameters(parser, ns_id, true, true)
+                .map(|mut ps| params.append(&mut ps)),
+            "return-value" => {
+                if ret.is_some() {
+                    return Err(parser.fail("Too many <return-value> elements"));
+                }
+                self.read_parameter(parser, ns_id, elem, true, false)
+                    .map(|p| ret = Some(p))
+            }
+            "source-position" => parser.ignore_element(),
+            "doc" => parser.text().map(|t| doc = Some(t)),
+            "doc-deprecated" => parser.text().map(|t| doc_deprecated = Some(t)),
+            "attribute" => parser.ignore_element(),
+            _ => Err(parser.unexpected_element(elem)),
+        })?;
+
+        let throws = elem.attr_bool("throws", false);
+        if throws {
+            params.push(Parameter {
+                name: "error".into(),
+                typ: self.find_or_stub_type(ns_id, "GLib.Error"),
+                c_type: "GError**".into(),
+                instance_parameter: false,
+                direction: ParameterDirection::Out,
+                transfer: Transfer::Full,
+                caller_allocates: false,
+                nullable: Nullable(true),
+                array_length: None,
+                is_error: true,
+                doc: None,
+                scope: ParameterScope::None,
+                closure: None,
+                destroy: None,
+            });
+        }
+
+        if let Some(ret) = ret {
+            Ok(Function {
+                name: method_name.into(),
+                c_identifier: c_identifier.map(|s| s.into()),
+                kind: FunctionKind::VirtualMethod,
+                parameters: params,
+                ret,
+                throws,
+                version,
+                deprecated_version,
+                doc,
+                doc_deprecated,
+            })
+        } else {
+            Err(parser.fail("Missing <return-value> element"))
+        }
+    }
+
     fn read_field(
         &mut self,
         parser: &mut XmlParser<'_>,
@@ -528,7 +625,7 @@ impl Library {
                 if typ.is_some() {
                     return Err(parser.fail("Too many <type> elements"));
                 }
-                self.read_function(parser, ns_id, elem.name(), elem)
+                self.read_function(parser, ns_id, elem.name(), elem, true)
                     .map(|f| {
                         typ = Some((Type::function(self, f), None, None));
                     })
@@ -559,7 +656,7 @@ impl Library {
         ns_id: u16,
         elem: &Element,
     ) -> Result<(), String> {
-        self.read_function_if_not_moved(parser, ns_id, elem.name(), elem)?
+        self.read_function_if_not_moved(parser, ns_id, elem.name(), elem, true)?
             .map(|func| {
                 let name = func.name.clone();
                 self.add_type(ns_id, &name, Type::Function(func))
@@ -583,6 +680,7 @@ impl Library {
         let deprecated_version = self.read_deprecated_version(parser, ns_id, elem)?;
 
         let mut fns = Vec::new();
+        let mut vfns = Vec::new();
         let mut signals = Vec::new();
         let mut properties = Vec::new();
         let mut prereqs = Vec::new();
@@ -606,7 +704,9 @@ impl Library {
             }),
             "doc" => parser.text().map(|t| doc = Some(t)),
             "doc-deprecated" => parser.text().map(|t| doc_deprecated = Some(t)),
-            "virtual-method" => parser.ignore_element(),
+            "virtual-method" => self
+                .read_virtual_method(parser, ns_id, elem)
+                .map(|v| vfns.push(v)),
             "source-position" => parser.ignore_element(),
             "attribute" => parser.ignore_element(),
             _ => Err(parser.unexpected_element(elem)),
@@ -619,6 +719,7 @@ impl Library {
             c_class_type: None, // this will be resolved during postprocessing
             glib_get_type: get_type.into(),
             functions: fns,
+            virtual_methods: vfns,
             signals,
             properties,
             prerequisites: prereqs,
@@ -738,7 +839,7 @@ impl Library {
         ns_id: u16,
         elem: &Element,
     ) -> Result<(), String> {
-        self.read_function_if_not_moved(parser, ns_id, "global", elem)
+        self.read_function_if_not_moved(parser, ns_id, "global", elem, false)
             .map(|func| {
                 if let Some(func) = func {
                     self.add_function(ns_id, func);
@@ -907,6 +1008,7 @@ impl Library {
         ns_id: u16,
         kind_str: &str,
         elem: &Element,
+        is_callback: bool,
     ) -> Result<Function, String> {
         let fn_name = elem.attr_required("name")?;
         let c_identifier = elem.attr("identifier").or_else(|| elem.attr("type"));
@@ -941,6 +1043,11 @@ impl Library {
             "attribute" => parser.ignore_element(),
             _ => Err(parser.unexpected_element(elem)),
         })?;
+        // The last argument of a callback is ALWAYS user data, so it has to be marked as such
+        // in case it's missing.
+        if is_callback && params.last().map(|x| x.closure.is_none()).unwrap_or(false) {
+            params.last_mut().unwrap().closure = Some(2000);
+        }
 
         let throws = elem.attr_bool("throws", false);
         if throws {
@@ -989,7 +1096,7 @@ impl Library {
         elem: &Element,
         fns: &mut Vec<Function>,
     ) -> Result<(), String> {
-        if let Some(f) = self.read_function_if_not_moved(parser, ns_id, elem.name(), elem)? {
+        if let Some(f) = self.read_function_if_not_moved(parser, ns_id, elem.name(), elem, false)? {
             fns.push(f);
         }
         Ok(())
@@ -1001,11 +1108,12 @@ impl Library {
         ns_id: u16,
         kind_str: &str,
         elem: &Element,
+        is_callback: bool,
     ) -> Result<Option<Function>, String> {
         if elem.attr("moved-to").is_some() {
             return parser.ignore_element().map(|_| None);
         }
-        self.read_function(parser, ns_id, kind_str, elem)
+        self.read_function(parser, ns_id, kind_str, elem, is_callback)
             .and_then(|f| {
                 if f.c_identifier.is_none() {
                     return Err(parser.fail_with_position(
@@ -1362,7 +1470,7 @@ impl Library {
                     self.register_version(ns_id, v);
                     Ok(Some(v))
                 }
-                Err(e) => Err(parser.fail(&format!("Invalid `{}` attribute: {}", attr, e))),
+                Err(e) => Err(parser.fail(&format!("Invalid `{attr}` attribute: {e}"))),
             }
         } else {
             Ok(None)
@@ -1387,7 +1495,7 @@ impl Library {
 
 fn make_file_name(dir: &Path, name: &str) -> PathBuf {
     let mut path = dir.to_path_buf();
-    let name = format!("{}.gir", name);
+    let name = format!("{name}.gir");
     path.push(name);
     path
 }
